//+------------------------------------------------------------------+
//|                                     SimpleDailyTrendReversal.mq4 |
//|                                    Copyright 2017, Erwin Beckers |
//|                                              www.erwinbeckers.nl |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, Erwin Beckers"
#property link      "www.erwinbeckers.nl"
#property version   "1.14"
#property strict


// 1.14
// - fixed issue that EA did not show valid signal due to MBFX
//
// 1.13
// - fixed: After a restart the EA set the stoploss at OrderSL instead of the tip of the zigzagarrow
// - EA now closes the order if another arrow appears
// - EA now shows open orders
//
// 1.12
// - EA now shows last update/refresh time so we can check that EA is still refreshing the dashboard
//
// 1.11
// - fixed news description overlapping with trade panel
//
// 1.10
// - fixed ea did not open when there was upcoming news for other pairs
// - new pair filter allows you to specify which pairs you want to trade
//
// 1.09
// - fixed error 4107 invalid price xxx for OrderSend function
// - fixed error 4107 invalid price xxx for OrderClose function
//
// 1.08
// - changed default for trendline and zigzag/mbfx candles
// - EA send same alert every minute
// - made MA15 more restrict  
// - use the Point & Digit values from the order-symbol when trailing stoploss
//
// 1.07
// - added news filter ( needs FFCal indicator to be installed)
// - fixed dashboard refresh issue
//
// 1.06
// - fixed bug with trailing stop 
// - fixed bug that EA was opening just 1 order even if there where valid signals on multiple pairs
// - dashboard: show buy/sell signal when all indicators are valid
// - cleaned up code
//
// 1.05
// - fixed bug dashboard was not refreshing somtimes when switching between timeframes
//
// 1.04
// - added (hidden) trailing stoploss/profit
// - fixed some false signals
//
// 1.03
// - added auto trading feature
// - added sending notifications to mobile and alerts to alert window
//
// 1.02
// - made it work on any timeframe
//
// 1.01
// - EA now only show pairs where setups are starting to happen
// - made moving average period and moving average type configurable
// - made # candles to look back for zigzag, mbfx, trendline configurable
// - fixed some small bugs
//
// 1.00
// - initial version

extern string      __trading                = " ------- trading settings ------------";
extern bool        allowTrading             = false;
extern bool        sendAlerts               = false;
extern bool        stopLossAtZigZagArrow    = true;
extern string      TradePairs               = "AUDCAD NZDUSD AUDUSD EURUSD USDCHF USDJPY USDCAD GBPUSD EURCHF EURGBP EURJPGY GPBAUD AUDNZD EURCAD EURNZD GBPCAD GBPNZD NZDCAD EURAUD AUDJPY CADCHF CADJPY NZDJPY AUDCHF CHFJPY GBPCHF GBPJPY NZDCHF";

#include <Erwin\COrders.mqh>
#include <Erwin\CTrailingStop.mqh>
#include <Erwin\CTimeFilter.mqh>
#include <Erwin\CNewsFilter.mqh>
#include <Erwin\CGui.mqh>
#include <Erwin\CInfoPanel.mqh>

extern string     __signals__          = " ------- Candles to look back for confirmation ------------";
extern int        ZigZagCandles        = 10;
extern int        MBFXCandles          = 5;

extern string     __movingaverage__    = " ------- moving average settings ------------";
extern int        MovingAveragePeriod  = 15;
extern int        MovingAverageType    = MODE_SMA;

#include <Erwin\CZigZag.mqh>
#include <Erwin\CMBFX.mqh>
#include <Erwin\CTrendLine.mqh>

CZigZag*          _zigZag;         
CMBFX*            _mbfx;         
CTrendLine*       _trendLine;        

COrders*         _orders;
CNewsFilter*     _newsFilter;
CTimeFilter*     _timeFilter;
CTrailingStop*   _trailingStop;

CInfoPanel*      _infoPanel = NULL;
CLabel*          _labelTitle;
CLabelKeyValue*  _labelBalance;
CLabelKeyValue*  _labelEquity;
CLabelKeyValue*  _labelFreeMargin;
CLabelKeyValue*  _labelMargin;
CLabelKeyValue*  _labelGlobalOrders;
CLabelKeyValue*  _labelGlobalProfit;

CLine*           _line1;
CLabelKeyValue*  _labelOrders ;
CLabelKeyValue*  _labelOrdersBuySell;
CLabelKeyValue*  _labelProfitBuySell;
CLabelKeyValue*  _labelProfit;

CLine*           _line3;
CLabelKeyValue*  _labelTotalTrades;
CLabelKeyValue*  _labelOrdersWonLost;

CLine*           _line4;
CLabelKeyValue*  _labelProfitToday;
CLabelKeyValue*  _labelProfitYesterday;
CLabelKeyValue*  _labelProfitAllTime;
CLabelKeyValue*  _labelProfitFactor;
CLabelKeyValue*  _labelExpectedPayOff;

CLine*           _line5;
CLabelKeyValue*  _labelTotalLotsTraded;
CLabelKeyValue*  _labelAverageLotsPerTrade;


//--------------------------------------------------------------------
void SetupPanel()
{
   _infoPanel               = new CInfoPanel(400, 360);
   
   _labelTitle              = new CLabel();
   _labelBalance            = new CLabelKeyValue("Balance",200);
   _labelEquity             = new CLabelKeyValue("Equity",200);
   _labelFreeMargin         = new CLabelKeyValue("Free margin",200);
   _labelMargin             = new CLabelKeyValue("Margin",200);
   _labelGlobalOrders       = new CLabelKeyValue("Global Open Orders",200);
   _labelGlobalProfit       = new CLabelKeyValue("Global Profit/Loss",200);
   _line1                   = new CLine();
   _labelOrders             = new CLabelKeyValue("Open Orders",200);
   _labelOrdersBuySell      = new CLabelKeyValue("       buy/sell",200);
   _labelProfitBuySell      = new CLabelKeyValue("   P/L buy/sell",200);
   _labelProfit             = new CLabelKeyValue("Profit/Loss",200);
   
   _line3                   = new CLine();
   _labelTotalTrades        = new CLabelKeyValue("Trades",200);
   _labelOrdersWonLost      = new CLabelKeyValue("Won/Lost",200);
   
   _line4                   = new CLine();
   _labelProfitToday        = new CLabelKeyValue("Profit today",200);
   _labelProfitYesterday    = new CLabelKeyValue("Profit yesterday",200);
   _labelProfitAllTime      = new CLabelKeyValue("All time profit",200);
   _labelProfitFactor       = new CLabelKeyValue("Profit factor",200);
   _labelExpectedPayOff     = new CLabelKeyValue("Expected payoff",200);
   
   _line5                   = new CLine();
   _labelTotalLotsTraded    = new CLabelKeyValue("Total lots traded",200);
   _labelAverageLotsPerTrade= new CLabelKeyValue("Average lots/trade",200);

   _labelTitle.Text="--- Trend Reversal EA v1.14 --- ";
   _labelTitle.FontSize = 10;   
   _line1.X = 0;
   _line1.Width = 400;
   _line3.X = 0;
   _line3.Width = 400;
   _line4.X = 0;
   _line4.Width = 400;
   _line5.X = 0;
   _line5.Width = 400;
   _infoPanel.SetPosition(720,20);
   _infoPanel.Add(_labelTitle, 80, 5);
   _infoPanel.Add(_labelBalance, 20, 10);
   _infoPanel.Add(_labelEquity, 20, 0);
   _infoPanel.Add(_labelFreeMargin, 20, 0);
   _infoPanel.Add(_labelMargin, 20, 0);
   _infoPanel.Add(_labelGlobalOrders, 20, 0);
   _infoPanel.Add(_labelGlobalProfit, 20, 0);
   _infoPanel.Add(_line1, 0, 5);
   _infoPanel.Add(_labelOrders, 20, -10);
   _infoPanel.Add(_labelOrdersBuySell, 20, 0);
   _infoPanel.Add(_labelProfitBuySell, 20, 0);
   _infoPanel.Add(_labelProfit, 20, 0);
   _infoPanel.Add(_line3, 0, 5);
   _infoPanel.Add(_labelTotalTrades, 20, -10);
   _infoPanel.Add(_labelOrdersWonLost, 20, 0);
   _infoPanel.Add(_line4, 0, 5);
   _infoPanel.Add(_labelProfitToday, 20, -10);
   _infoPanel.Add(_labelProfitYesterday, 20, 0);
   _infoPanel.Add(_labelProfitAllTime, 20, 0);
   _infoPanel.Add(_labelProfitFactor, 20, 0);
   _infoPanel.Add(_labelExpectedPayOff, 20, 0);
   _infoPanel.Add(_line5, 0, 5);
   _infoPanel.Add(_labelTotalLotsTraded, 20, -10);
   _infoPanel.Add(_labelAverageLotsPerTrade,20, 0);
   _infoPanel.Update();  
}

static int _line=0;

//--------------------------------------------------------------------
void Clear()
{
   for (int line=0; line < 50;++line)
   {
      for (int col=0; col < 50;++col)
      {
         string id="l"+IntegerToString(line)+"c"+IntegerToString(col);
         ObjectDelete(id);
      }
   }
}

//--------------------------------------------------------------------
void Draw(int line, int column, string text, color clr, int xoff=0)
{
   string id="l"+IntegerToString(line)+"c"+IntegerToString(column);
   double x= column==0 ?  20 : (column*70+35);
   if (_line ==0 && column > 0) x=x-10;
   ObjectCreate(0,id,OBJ_LABEL,0,0,0);
   ObjectSetInteger(0,id,OBJPROP_XDISTANCE, ((int)x)+xoff);
   ObjectSetInteger(0,id,OBJPROP_YDISTANCE,_line*17+20);
   ObjectSetString(0,id,OBJPROP_TEXT,text);
   ObjectSetString(0,id,OBJPROP_FONT,"Courier New");
   ObjectSetInteger(0,id,OBJPROP_ANCHOR,ANCHOR_LEFT_UPPER);
   ObjectSetInteger(0,id,OBJPROP_FONTSIZE,10);
   ObjectSetInteger(0,id,OBJPROP_COLOR,clr);
}

void DrawWingDings(int line, int column, string text, color clr, int xoff=0)
{
   string id="l"+IntegerToString(line)+"c"+IntegerToString(column);
   double x= column==0 ?  20 : (column*70+30);
   ObjectCreate(0,id,OBJ_LABEL,0,0,0);
   ObjectSetInteger(0,id,OBJPROP_XDISTANCE, ((int)x)+xoff );
   ObjectSetInteger(0,id,OBJPROP_YDISTANCE,_line*17+23);
   ObjectSetString(0,id,OBJPROP_TEXT,text);
   ObjectSetString(0,id,OBJPROP_FONT,"WingDings");
   ObjectSetInteger(0,id,OBJPROP_ANCHOR,ANCHOR_LEFT_UPPER);
   ObjectSetInteger(0,id,OBJPROP_FONTSIZE,10);
   ObjectSetInteger(0,id,OBJPROP_COLOR,clr);
}


//--------------------------------------------------------------------
void DrawSymbol( string symbol,
                 bool   zigZagOk,
                 string zigZag, 
                 color  zigZagColor,
                 bool   mbfxOk,
                 bool   trendOk,
                 bool   maOk,
                 bool   buy,
                 bool   sell,
                 bool   allowedToTradeTimeFilter,
                 bool   allowedToTradeNews,
                 string news,
                 NEWS_IMPACT impact
               )
{
   if (zigZagOk)
   {
      color newsColor=White;
      if (impact==IMPACT_HIGH) newsColor=Red;
      if (impact==IMPACT_MEDIUM) newsColor=Orange;
      if (impact==IMPACT_LOW) newsColor=Green;
      
      Draw(_line,0,TimeToStr(TimeCurrent(), TIME_MINUTES),White);
      Draw(_line,1,symbol, White,-30);
      DrawWingDings(_line, 2,zigZag, zigZagColor,-20);
      if (zigZag != "")
      {
         if (mbfxOk)  DrawWingDings(_line,3,CharToStr(252), zigZagColor,-20);
         else Draw(_line, 3,"-", White,-20);
         
         if (trendOk) DrawWingDings(_line,4,CharToStr(252), zigZagColor,-20);
         else Draw(_line, 4,"-", White,-20);
     
         if (maOk) DrawWingDings(_line,5,CharToStr(252), zigZagColor,-20);
         else Draw(_line, 5,"-", White,-20);
         
         
         if (!allowedToTradeTimeFilter) 
         {
            Draw(_line, 6,"Time", Red,-30);
         }
         else if (!allowedToTradeNews)
         {
            Draw(_line, 6,"X", Red,-20);
         }
         else if (news!="")
         {
            Draw(_line, 6,news, newsColor, -40); 
         }
         else 
         {
            Draw(_line, 6,"-", White, -20); 
         }
         
         if (mbfxOk && trendOk && maOk && buy)  Draw(_line,7,"<- Buy " + symbol, zigZagColor,-30);
         else if (mbfxOk && trendOk && maOk && sell)  Draw(_line,7,"<- Sell  " + symbol, zigZagColor,-30);
         else Draw(_line,7,"", White,-30); 
         
      }
      _line++;
   }
   
}

//--------------------------------------------------------------------
// https://www.forexfactory.com/showthread.php?t=713593
//--------------------------------------------------------------------
void OnSymbol(string symbol)
{
   _zigZag.Refresh(symbol);
   _mbfx.Refresh(symbol);
   _trendLine.Refresh(symbol);
   
   string zigArrow     = " ";
   color  zigArrowColor= Green;
   bool   zigZagBuy    = false;
   bool   zigZagSell   = false;
   int    zigZagBar    = -1;
   bool   mbfxOk  = false;
   bool   trendOk = false;
   bool   maOk    = false;
   
   // Rule #1: a zigzar arrow appears
   for (int bar = ZigZagCandles; bar >= 1;bar--)
   {
      ARROW_TYPE arrow = _zigZag.GetArrow(bar); 
      if (arrow == ARROW_BUY)
      {
         zigArrow     = CharToString(233);
         zigArrowColor= Green;
         zigZagBuy    = true;
         zigZagSell   = false;
         zigZagBar    = bar;
      }
      if (arrow == ARROW_SELL)
      {
         zigArrow     = CharToString(234);
         zigArrowColor= Red;
         zigZagBuy    = false;
         zigZagSell   = true;
         zigZagBar    = bar;
      }
   }
      
   // BUY signals
   if (zigZagBuy && zigZagBar>0)
   {
      // MBFX should be green at the moment
      if (_mbfx.IsGreen(1))
      { 
         // and should have been below < 30 some candles ago
         int barStart=zigZagBar;
         if (zigZagBar==1) barStart=2;
         for (int bar=MathMin(barStart, MBFXCandles); bar >=1; bar--)
         {
            double red   = _mbfx.RedValue(bar);
            double green = _mbfx.GreenValue(bar);
            if (red < 30 || green < 30)
            {
               mbfxOk=true;
            }
         }
      }
            
      // trend line should be green at the moment
      if (_trendLine.IsGreen(1))
      {
         trendOk = true;
      }
      
      // rule #4: price should be above 15 SMA 
      double ma1 = iMA(symbol,0, MovingAveragePeriod, 0, MovingAverageType, PRICE_CLOSE, 1);
      double ma2 = iMA(symbol,0, MovingAveragePeriod, 0, MovingAverageType, PRICE_CLOSE, 2);
      if ( (  ( iClose(symbol, 0, 1) > ma1 && iOpen(symbol,0,1) < ma1) ||
              ( iClose(symbol, 0, 2) > ma2 && iOpen(symbol,0,2) < ma2) 
           )  
           && iClose(symbol, 0, 1) > ma1  ) 
      {
         maOk=true;
      }
   }
   
   
   // SELL signals
   if (zigZagSell && zigZagBar>0)
   {
      // MBFX should now be red
      if (!_mbfx.IsGreen(1))
      { 
         // and should been above > 70 some candles ago
         int barStart=zigZagBar;
         if (zigZagBar==1) barStart=2;
         for (int bar=MathMin(barStart, MBFXCandles); bar >=1; bar--)
         {
            double red   = _mbfx.RedValue(bar);
            double green = _mbfx.GreenValue(bar);
            if (red > 70)
            {
               mbfxOk=true;
            }
         }
      }
      
      // trend line should now be red 
      if (_trendLine.IsRed(1))
      {
         trendOk = true;
      }
            
      // rule #4: and price below SMA15 on previous candle
      double ma1 = iMA(symbol,0, MovingAveragePeriod, 0, MovingAverageType, PRICE_CLOSE, 1);
      double ma2 = iMA(symbol,0, MovingAveragePeriod, 0, MovingAverageType, PRICE_CLOSE, 2);
      if (  ( (iClose(symbol, 0, 1) < ma1 && iOpen(symbol,0,1) > ma1) ||
              (iClose(symbol, 0, 2) < ma2 && iOpen(symbol,0,2) > ma2) 
            ) && iClose(symbol, 0, 1) < ma1 )
      {
        maOk=true;
      }
   }
   
   string      news = "";
   NEWS_IMPACT impact;
   bool allowedToTradeNews = _newsFilter.GetNews(symbol, news, impact);  
   bool allowedToTradeTimeFilter = _timeFilter.CanTrade();
   DrawSymbol(symbol, zigZagBar>=1, zigArrow, zigArrowColor, mbfxOk, trendOk, maOk,zigZagBuy,zigZagSell, allowedToTradeTimeFilter, allowedToTradeNews, news, impact);
   
   // close orders in profit when news is arriving..
   if (!allowedToTradeNews)
   {
      if (CloseProfitableTradesBeforeNews)
      {
         bool closedOrders = _orders.CloseOrdersInProfit(symbol);
         if (closedOrders) Print(symbol, " Closed order because of upcoming news.");
      }
   }
   
   // send alerts enabled ?
   if (sendAlerts)
   {
      // valid signal ?
      if (zigZagBar >= 1 && mbfxOk && trendOk && maOk)
      {
          if (zigZagBuy)
          {
            // send buy alert
            _utils.SendNotify(symbol, " buy signal at " + _utils.GetTimeFrame() );
          }
          else if (zigZagSell)
          {
            // send sell alert
           _utils.SendNotify(symbol, " sell signal at " + _utils.GetTimeFrame() );
          }
          else
          {
            _utils.SendNotify(symbol,"");
          }
      }
      else
      {
        _utils.SendNotify(symbol,"");
      }
   }
   
   // trading enabled ?
   if (allowTrading)
   {
      // do we have a zigzag arrow ?
      if (zigZagBar >= 1 )
      {
         // if it is a buy signal, then close any sell orders
         if (zigZagBuy)
         {  
            _orders.CloseOrderByType(symbol, OP_SELL);
         }
         
         // if it is a sell signal, then close any buy orders
         if (zigZagSell)
         {
            _orders.CloseOrderByType(symbol, OP_BUY);
         }
      }
      
      // is it a valid signal ?
      if (zigZagBar >= 1 && mbfxOk && trendOk && maOk)
      {
         //Print(symbol, " Detected a valid signal bar:", zigZagBar, " buy:", zigZagBuy," sell:", zigZagSell);
         // allowed to trade at the moment ?
         if (allowedToTradeTimeFilter)
         {
            if (allowedToTradeNews)
            {
               //Print(symbol, " -> time filter ok");
               // no trade running for this symbol yet ?
               int orderCount = _orders.GetOrderCount(symbol);
               if (orderCount == 0)
               {
                  // no order running atm
                  // get last order for this symbol
                  COrder* order = _orders.GetLastOrder(symbol);
                  if (order != NULL)
                  {
                     // check #minutes elapsed since this order has closed
                     double timeElapsed =(double)(TimeCurrent() - order.CloseTime);
                     delete order;
                     timeElapsed /= 60.0;
                     
                     //  Print(symbol," last order opened at ", order.OpenTime, " and closed at ", order.CloseTime, " which is ", timeElapsed," ago", " period:", Period());
                     if (timeElapsed <= Period())
                     {  
                         //Print(symbol,"   So don't open new order", timeElapsed, "  ",TimeCurrent());
                         delete order;
                         return;
                     }
                     delete order;
                  }
                  
                  // then place order..
                  if (zigZagBuy)
                  {  
                     // buy order
                     Print(symbol, " -> place buy order");
                     double sl = iLow(symbol, 0, zigZagBar);
                     double price=MarketInfo(symbol, MODE_ASK);
                     double digits=MarketInfo(symbol, MODE_DIGITS);
                     int ticket = _orders.OpenBuyOrder(symbol, _orders.GetLotSize(symbol), 0, 0);
                     if ( stopLossAtZigZagArrow && ticket >= 0)
                     {
                        _trailingStop.SetInitalStoploss(ticket, sl);
                     }
                  }
                  else if (zigZagSell)
                  {
                     // sell order
                     Print(symbol, " -> place sell order");
                     double sl = iHigh(symbol, 0, zigZagBar);
                     double price=MarketInfo(symbol, MODE_BID);
                     double digits=MarketInfo(symbol, MODE_DIGITS);
                     int ticket =_orders.OpenSellOrder(symbol, _orders.GetLotSize(symbol), 0, 0);
                     
                     if ( stopLossAtZigZagArrow && ticket >= 0)
                     {
                        _trailingStop.SetInitalStoploss(ticket, sl);
                     }
                  }
               }
               else 
               {
                  //Print(symbol, " -> already has an order running");
               }
            }
            else 
            {
               //Print(symbol, " -> not allowed to trade due to news filter");
            }
         }
         else 
         {
           //Print(symbol, " -> not allowed to trade due to time filter");
         }
      } // if valid signal
   } // if allow trading
}

//--------------------------------------------------------------------
void OnNewBar()
{
   Clear();
   _line=0;
   Draw(_line,0,"Last update: "+TimeToStr(TimeCurrent()),White);
   
   _line++;
   Draw(_line,0,"Time", White);
   Draw(_line,1,"Pair", White,-30);
   Draw(_line,2,"ZigZag", White,-35);
   Draw(_line,3,"MBFX", White,-30);
   Draw(_line,4,"Trend", White,-30);
   Draw(_line,5,"MA"+IntegerToString(MovingAveragePeriod), White,-30);
   Draw(_line,6,"News", White,-30);
   Draw(_line,7,"Signal", White,-30);
   _line++;

   if (IsTesting() || IsOptimization() )
   {
      allowTrading = true;
      OnSymbol(Symbol());
      return;
   }
   
   
   string pairs[30];
   int pairCount=  StringSplit(TradePairs,StringGetCharacter(" ",0),pairs);
   for (int i=0; i <  SymbolsTotal(true);++i)
   {
      string symbol = SymbolName(i,true);
      for (int x=0; x < pairCount;++x)
      {
         if (StringFind(symbol, pairs[x])>=0)
         {
            OnSymbol(symbol);
            break;
         }
      }
   }
  
   _line++;
   Draw(_line,0,"Opened",White);
   Draw(_line,2,"Symbol",White);
   Draw(_line,3,"Price",White);
   Draw(_line,4,"Price",White,20);
   Draw(_line,5,"S/L",White,40);
   Draw(_line,6,"Profit",White,60);
   Draw(_line,7,"Pips",White,80);
   _line++;
   // show open orders
   for (int i=0; i < OrdersTotal();++i)
   {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
         {
            string symbol   = OrderSymbol();
            double profit   = OrderProfit() + OrderCommission() + OrderSwap();
            double pips     = 0;
            double price    = MarketInfo(symbol, MODE_ASK);
            double points   = MarketInfo(symbol, MODE_POINT);
            double digits   = MarketInfo(symbol, MODE_DIGITS);
            double SL       = _trailingStop.GetStoploss( OrderTicket() );
            double mult = 1;
            if (digits ==3 || digits==5) mult = 10;
            if (OrderType() == OP_BUY)
            {
               pips = ( price - OrderOpenPrice()) ;
            }
            else
            {
               pips = ( OrderOpenPrice() - price);
            }
            if (pips != 0)
            {
               pips /= mult;
               pips /= points;
            }
            Draw(_line,0, TimeToStr(OrderOpenTime()), White,0);
            Draw(_line,2, symbol  , White,0);
            Draw(_line,3, DoubleToString(OrderOpenPrice(),5)  , White,0);
            Draw(_line,4, DoubleToString(MarketInfo(symbol,MODE_ASK),5)  , White,20);
            Draw(_line,5,DoubleToString(SL,5) , White, 40);
            Draw(_line,6,DoubleToString(profit,2) , profit > 0 ? Green:Red,60);
            Draw(_line,7,DoubleToString(pips,2) , pips > 0 ? Green:Red,80);
            
            _line++;
         }
      }
   }
}

//--------------------------------------------------------------------
void UpdatePanel()
{
   if (IsOptimization()) return;  
   if (IsTesting()) return;
  
   _labelBalance.Text    =  DoubleToString(AccountBalance(),2) + " " + AccountCurrency();
   _labelEquity.Text     =  DoubleToString(AccountEquity(),2)  + " " + AccountCurrency();
   _labelFreeMargin.Text =  DoubleToString(AccountFreeMargin(),2) + " " + AccountCurrency();
   _labelMargin.Text     =  DoubleToString(AccountMargin(),2) + " " + AccountCurrency();
   
   _labelGlobalProfit.Text  =  DoubleToString( _orders.GlobalProfit,2) +" " + AccountCurrency();
   _labelGlobalProfit.Color =  _orders.GlobalProfit >= 0 ? White:Red;
   _labelGlobalOrders.Text  =  IntegerToString(_orders.GlobalOrderCount) ;
   
   _labelProfit.Text        =  DoubleToString( _orders.TotalProfit,2) +" " + AccountCurrency();
   _labelProfit.Color       =  _orders.TotalProfit >= 0 ? White:Red;
   _labelOrders.Text        =  IntegerToString(_orders.TotalCount);
   _labelOrdersBuySell.Text = " buy:" + IntegerToString(_orders.BuyOrders) + " / sell: " + IntegerToString(_orders.SellOrders) ;
   _labelProfitBuySell.Text = DoubleToString(_orders.TotalProfitOfBuyOrders,2)  +" " + AccountCurrency() + " / " + DoubleToString(_orders.TotalProfitOfSellOrders, 2)  + " " + AccountCurrency();
   
   _labelOrdersWonLost.Text  = IntegerToString(_orders.OrdersWon) + " / " + IntegerToString(_orders.OrdersLost) + " (" + DoubleToString(_orders.WinPercentage, 2) + " % winrate)";
   _labelOrdersWonLost.Color = (_orders.OrdersWon + _orders.OrdersLost) >= 0 ? White:Red; 
   
   _labelTotalTrades.Text  = IntegerToString(_orders.TotalTrades);
   _labelProfitToday.Text  = DoubleToStr(_orders.ProfitToday, 2) + " " + AccountCurrency();
   _labelProfitToday.Color = (_orders.ProfitToday) >= 0 ? White:Red; 
   
   _labelProfitYesterday.Text  = DoubleToStr(_orders.ProfitYesterday, 2) + " " + AccountCurrency();
   _labelProfitYesterday.Color = (_orders.ProfitYesterday) >= 0 ? White:Red; 
   
   _labelProfitAllTime.Text  = DoubleToStr(_orders.AllTimeProfit, 2) + " " + AccountCurrency();
   _labelProfitAllTime.Color = (_orders.AllTimeProfit) >= 0 ? White:Red; 
   
   _labelProfitFactor.Text   = DoubleToStr(_orders.ProfitFactor, 2);    
   _labelExpectedPayOff.Text = DoubleToStr(_orders.ExpectedPayOff, 2)+ " " + AccountCurrency() + " / trade";
   
   _labelTotalLotsTraded.Text     = DoubleToStr(_orders.TotalLotsTraded, 3) + " lots";
   _labelAverageLotsPerTrade.Text = DoubleToStr(_orders.AverageLotsPerTrade, 3) + " lots";
   _infoPanel.Update();
}

//--------------------------------------------------------------------
void SetStoplossOnOpenOrders()
{
   if (false == stopLossAtZigZagArrow) return;
    
   for (int i=0; i < OrdersTotal();++i)
   {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
         {
            string symbol = OrderSymbol();
            _zigZag.Refresh(symbol);
            
            // find last zigzag arrow
            int zigZagBar = -1;
            ARROW_TYPE arrow = ARROW_NONE;
            for (int bar=1; bar < 200;++bar)
            {
               arrow = _zigZag.GetArrow(bar);
               if (arrow == ARROW_BUY || arrow == ARROW_SELL)
               {
                  zigZagBar=bar;
                  break;
               }
            }
            if (zigZagBar > 0)
            {
               if (arrow == ARROW_BUY)
               {
                  double sl = iLow(symbol, 0, zigZagBar);
                  Print(symbol, " -> order ",OrderTicket()," found:, set virtual SL to ", sl);
                  _trailingStop.SetInitalStoploss(OrderTicket(), sl);
               }
               else if (arrow == ARROW_SELL)
               {
                  double sl = iHigh(symbol, 0, zigZagBar);
                  Print(symbol, " -> order ",OrderTicket()," found:, set virtual SL to ", sl);
                  _trailingStop.SetInitalStoploss(OrderTicket(), sl);
               }
            }
         }
      }
   }
}

//--------------------------------------------------------------------
int OnInit()
{
   Print("--- Simple Daily Trend Reversal ---");
   Clear();
   _orders       = new COrders(false);
   _timeFilter   = new CTimeFilter();
   _newsFilter   = new CNewsFilter(_orders);
   _trailingStop = new CTrailingStop(_orders);
   _utils        = new CUtils();
   _zigZag       = new CZigZag();
   _mbfx         = new CMBFX();
   _trendLine    = new CTrendLine();
   _line         = 0;
      
   OnNewBar();
   SetStoplossOnOpenOrders();
   
   if (allowTrading)
   {
     SetupPanel();
     UpdatePanel();
   }
   return(INIT_SUCCEEDED);
}

//--------------------------------------------------------------------
void OnDeinit(const int reason)
{
   Print("--- Simple Daily Trend Reversal deinit---");
   Clear();
   delete _utils;
   delete _orders;
   delete _timeFilter;
   delete _trailingStop;
   delete _zigZag;
   delete _mbfx;
   delete _trendLine;
   delete _newsFilter;
   if (_infoPanel!=NULL) delete _infoPanel;
   _infoPanel=NULL;
}

//--------------------------------------------------------------------
void OnTick()
{ 
  static int lastMinute=-1;
   // manage orders
   _newsFilter.Check();
   _utils.Refresh();
   _orders.Refresh();
   _trailingStop.Trail();  
   
   // update dashboard when new bar occured'
   int min = TimeMinute(TimeCurrent());
   if (min != lastMinute)
   {
      lastMinute = min;
      OnNewBar();
   }
   
   // update trading panel
   if (allowTrading)
   {
     UpdatePanel();
   }
}
