//+------------------------------------------------------------------+
//|                                                       CTrend.mqh |
//|                                    Copyright 2017, Erwin Beckers |
//|                                              www.erwinbeckers.nl |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, Erwin Beckers"
#property link      "www.erwinbeckers.nl"
#property strict

enum ENUM_ENTRY_METHOD
{
   EMA_Slope, 
   Close_And_EMA_200
};

enum ENUM_TREND_DIRECTION
{
   TREND_UP,
   TREND_DOWN,
   TREND_CONSILIDATION
};

extern ENUM_ENTRY_METHOD	Entry_Method				= EMA_Slope;
extern string				___Parameter_EMA_Slope		= "-------------------------------------------";
extern int					   ma_period					= 32;
extern ENUM_MA_METHOD		ma_method					= MODE_EMA;
extern ENUM_APPLIED_PRICE	applied_price				= PRICE_TYPICAL;
extern ENUM_TIMEFRAMES		timeFrameDirection		= PERIOD_M15;

extern string				___Parameter_EMA_200		   = "-------------------------------------------";
extern int					   ema_period					= 200;
extern ENUM_APPLIED_PRICE	ema_price					= PRICE_CLOSE;
extern ENUM_TIMEFRAMES		ema_timeFrameDirection	= PERIOD_H1;


class CTrendDetector
{
public:
   // + ------------------------------------------------------------------ + 
   // + ------------------------------------------------------------------ + 
   ENUM_TREND_DIRECTION GetDirection()
   {
   	ENUM_TREND_DIRECTION direction = TREND_CONSILIDATION;
   
   	//-- entry PARACELSUS -----------------------------------------------
   	if (Entry_Method == Close_And_EMA_200)
   	{
   		double dClose1 = iClose(Symbol(), ema_timeFrameDirection, 1);
   		double dEMA1   = iMA(Symbol(), ema_timeFrameDirection, ema_period, 0, MODE_EMA, ema_price, 1);
   		if (dClose1 >= dEMA1) 	  direction = TREND_UP;
   		else if (dClose1 < dEMA1) direction = TREND_DOWN;
   	}
   	else if (Entry_Method == EMA_Slope)
   	{
   		//-- entry SnowBall -------------------------------------------------  
   		int signalBar = 1;
   
   		double avg10 = iMA(Symbol(), timeFrameDirection, ma_period, 0, ma_method, applied_price, signalBar);
   		double avg11 = iMA(Symbol(), timeFrameDirection, ma_period, 1, ma_method, applied_price, signalBar);
   		double diff10 = 10000 * (avg10 - avg11) / avg10;
   
   		double avg20 = iMA(Symbol(), timeFrameDirection, ma_period, 0, ma_method, applied_price, signalBar + 1);
   		double avg21 = iMA(Symbol(), timeFrameDirection, ma_period, 1, ma_method, applied_price, signalBar + 1);
   		double diff20 = 10000 * (avg20 - avg21) / avg20;
   
   		double avg30 = iMA(Symbol(), timeFrameDirection, ma_period, 0, ma_method, applied_price, signalBar + 2);
   		double avg31 = iMA(Symbol(), timeFrameDirection, ma_period, 1, ma_method, applied_price, signalBar + 2);
   		double diff30 = 10000 * (avg30 - avg31) / avg30;
   
   		double avg40 = iMA(Symbol(), timeFrameDirection, ma_period, 0, ma_method, applied_price, signalBar + 3);
   		double avg41 = iMA(Symbol(), timeFrameDirection, ma_period, 1, ma_method, applied_price, signalBar + 3);
   		double diff40 = 10000 * (avg40 - avg41) / avg40;
   
   		double trigger1 = (diff10 + diff20 + diff30) / 3.0;
   		double trigger2 = (diff20 + diff30 + diff40) / 3.0;
   
   		//-- trend direction; condition for two bars
   		if ( (diff10 > 0 && diff10 > trigger1) || (diff20 > 0 && diff20 > trigger2)) direction = TREND_UP; 
   		else if ( (diff10 < 0 && diff10 < trigger1) || (diff20 < 0 && diff30 < trigger2)) direction = TREND_DOWN;
   	}
   	return direction;
   }
};