//+------------------------------------------------------------------+
//|                                                      CZigZag.mqh |
//|                                    Copyright 2017, Erwin Beckers |
//|                                              www.erwinbeckers.nl |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, Erwin Beckers"
#property link      "www.erwinbeckers.nl"
#property strict


// inputs for zigzag
extern string     __zigzag             = " ------- ZigZag settings------------";
extern int        ExtDepth             = 60;
extern int        ExtDeviation         =  5;
extern int        ExtBackstep          =  3; 


enum ARROW_TYPE
{
   ARROW_NONE,
   ARROW_BUY,
   ARROW_SELL
};

class CZigZag
{
private: 
   double            _zigZagBufferBuy[];
   double            _zigZagBufferSell[];
   
public: 
   CZigZag()
   {
      ArrayResize(_zigZagBufferBuy,505,0);
      ArrayResize(_zigZagBufferSell,505,0);
   }
   
   ~CZigZag()
   {
   }
   
   void Refresh(string symbol)
   {      
      int    shift, back,lasthighpos,lastlowpos;
      double val,res;
      double curlow,curhigh,lasthigh,lastlow;
      lastlow=0;
      lasthigh=0;
   
      ArrayInitialize(_zigZagBufferBuy,  0);
      ArrayInitialize(_zigZagBufferSell, 0);
      for(shift=500-ExtDepth; shift>=0; shift--)
      {
         val=iLow(symbol, 0, iLowest(symbol, 0,MODE_LOW,ExtDepth,shift));
         if(val==lastlow) val=0.0;
         else 
         { 
            lastlow=val; 
            if(( iLow(symbol, 0, shift)-val)>(ExtDeviation*Point)) val=0.0;
            else
            {
               for(back=1; back<=ExtBackstep; back++)
               {
                  res=_zigZagBufferBuy[shift+back];
                  if((res!=0)&&(res>val)) _zigZagBufferBuy[shift+back]=0.0; 
               }
            }
         } 
         _zigZagBufferBuy[shift]=val;
         //--- high
         val=iHigh(symbol,0, iHighest(symbol, 0,MODE_HIGH,ExtDepth,shift));
         if(val==lasthigh) val=0.0;
         else 
         {
            lasthigh=val;
            if((val-iHigh(symbol,0, shift))>(ExtDeviation*Point)) val=0.0;
            else
            {
               for(back=1; back<=ExtBackstep; back++)
               {
                  res=_zigZagBufferSell[shift+back];
                  if((res!=0)&&(res<val)) _zigZagBufferSell[shift+back]=0.0; 
               } 
            }
         }
         _zigZagBufferSell[shift]=val;
      }
   
      // final cutting 
      lasthigh=-1; lasthighpos=-1;
      lastlow=-1;  lastlowpos=-1;
   
      for(shift=500-ExtDepth; shift>=0; shift--)
      {
         curlow =_zigZagBufferBuy[shift];
         curhigh=_zigZagBufferSell[shift];
         if((curlow==0)&&(curhigh==0)) continue;
         //---
         if(curhigh!=0)
         {
            if(lasthigh>0) 
            {
               if(lasthigh<curhigh) _zigZagBufferSell[lasthighpos]=0;
               else _zigZagBufferSell[shift]=0;
            }
            //---
            if(lasthigh<curhigh || lasthigh<0)
            {
               lasthigh=curhigh;
               lasthighpos=shift;
            }
            lastlow=-1;
         }
         //----
         if(curlow!=0)
         {
            if(lastlow>0)
            {
               if(lastlow>curlow) _zigZagBufferBuy[lastlowpos]=0;
               else _zigZagBufferBuy[shift]=0;
            }
            //---
            if((curlow<lastlow)||(lastlow<0))
            {
               lastlow=curlow;
               lastlowpos=shift;
            } 
            lasthigh=-1;
         }
      }
     
      for(shift=500-1; shift>=0; shift--)
      {
         if(shift>=500-ExtDepth) _zigZagBufferBuy[shift]=0.0;
         else
         {
            res=_zigZagBufferSell[shift];
            if(res!=0.0) _zigZagBufferSell[shift]=res;
         }
      }
   }
   
   ARROW_TYPE GetArrow(int bar)
   {
      if (bar < 0 || bar >=500) return ARROW_NONE;
      if ( _zigZagBufferBuy[bar] !=0 ) return ARROW_BUY;
      if ( _zigZagBufferSell[bar] !=0 ) return ARROW_SELL;
      return ARROW_NONE;
   }
};